SuperStrict

' miniBCC 23 - Farbwhler
' Beitrag von Holzchopf

Framework MaxGUI.Drivers
Import BRL.Timer
Import BRL.Retro
Import BRL.EventQueue
Import BRL.GLMax2D

Const RED:Int	= 0
Const GRN:Int	= 1
Const BLU:Int	= 2
Const SEL_RAD:Int		= 5			' radius of selector
Const SEL_SIZE:Int	= 2 *SEL_RAD +1
Const SEL_OUTER:Int	= $ff000000		' color of outer border of selector
Const SEL_INNER:Int	= $ffffffff		' color of inner border of selector
Const FIELD_SIZE:Int	= 256			' size of field (only image)
Const FIELD_MAX:Int	= FIELD_SIZE -1

' app window
Global gdWindow:TGadget = CreateWindow("Color Mixer", 0,0, 440,336, Null, WINDOW_TITLEBAR | WINDOW_CENTER | WINDOW_CLIENTCOORDS )
' color field
Global gdField:TGadget = CreateCanvas(40,40, 256,256, gdWindow)
Global imgField:TImage = CreateImage(FIELD_SIZE, FIELD_SIZE)
SetGadgetSensitivity(gdField, SENSITIZE_ALL)
' base colors
Global bcol:Int[][] = [[255,255,255],[0,255,255],[0,0,0],[0,0,255]]
Global gdBCol:TGadget[4]
For Local i:Int = 0 Until 4
	Local x:Int = 10 +(i Mod 2) *286
	Local y:Int = 10 +(i /2) *286
	gdBCol[i] = CreateButton("", x,y, 30,30, gdWindow)
	SetGadgetColor(gdBCol[i], bcol[i][RED], bcol[i][GRN], bcol[i][BLU])
Next
' color indicator (panel)
Global gdColInd:TGadget = CreatePanel(370,10, 60,210, gdWindow, PANEL_SUNKEN)
' color indicator (text)
Global gdCol:TGadget[4]
For Local i:Int = 0 Until 4
	Local y:Int = 231 +i *25
	CreateLabel(["R","G","B","HEX"][i], 336, y+2, 30,20, gdWindow, LABEL_RIGHT)
	gdCol[i] = CreateTextField(370,y, 60,20, gdWindow)
	' win only: make gadget read-only
	?Win32
		Local hwnd:Int = QueryGadget(gdCol[i], QUERY_HWND)
		SendMessageW(hwnd, EM_SETREADONLY, True, 0)
	?
Next
' color selector
Global imgSel:TImage = CreateImage(SEL_SIZE, SEL_SIZE)
SetImageHandle(imgSel, SEL_RAD, SEL_RAD)
Global selx:Int=200, sely:Int=64, selhold:Int, selrgb:Int

' initialize app
DrawSelector()
DrawField()
UpdateCol()
CreateTimer(25)

Repeat
	WaitEvent()
	Select EventID()
		' keep graphic up to date
		Case EVENT_TIMERTICK
			RedrawGadget(gdField)
		' quit app
		Case EVENT_WINDOWCLOSE
			End
		' paint field and selector
		Case EVENT_GADGETPAINT
			SetGraphics CanvasGraphics(gdField)
			SetViewport(0,0, FIELD_SIZE,FIELD_SIZE)
			SetBlend SOLIDBLEND
			DrawImage(imgField, 0,0)
			SetBlend ALPHABLEND
			DrawImage(imgSel, selx,sely)
			Flip
		' gadget (button) interaction
		Case EVENT_GADGETACTION
			' change base colors
			For Local i:Int = 0 Until 4
				If EventSource() = gdBCol[i] Then
					' request new color
					If RequestColor(bcol[i][RED], bcol[i][GRN], bcol[i][BLU]) Then
						bcol[i][RED] = RequestedRed()
						bcol[i][GRN] = RequestedGreen()
						bcol[i][BLU] = RequestedBlue()
						SetGadgetColor(gdBCol[i], bcol[i][RED], bcol[i][GRN], bcol[i][BLU])
					EndIf
					' update field
					DrawField()
					UpdateCol()
				EndIf
			Next
		' place selector / start dragging
		Case EVENT_MOUSEDOWN
			selhold = True
			selx = EventX()
			sely = EventY()
		' stop dragging
		Case EVENT_MOUSEUP, EVENT_APPSUSPEND
			selhold = False
		' drag selector around
		Case EVENT_MOUSEMOVE
			If selhold Then
				selx = EventX()
				sely = EventY()
			EndIf
?Debug
		Default
			Print CurrentEvent.ToString()
?
	End Select
	' update colors if selector was placed or dragged
	If selhold Then
		UpdateCol()
	EndIf
Forever
End

' generate field image
Function DrawField()
	Local pm:TPixmap = LockImage(imgField)
	' go through image
	For Local y:Int = 0 Until FIELD_SIZE
		For Local x:Int = 0 Until FIELD_SIZE
			' interpolate color and draw pixel
			Local rgb:Int = InterpolCol(x,y)
			WritePixel(pm, x,y, rgb)
		Next
	Next
	UnlockImage(imgField)
End Function

' generate selector image
Function DrawSelector()
	Local pm:TPixmap = LockImage(imgSel)
	' go through image
	For Local y:Int = 0 Until SEL_SIZE
		For Local x:Int = 0 Until SEL_SIZE
			' distance of pixel to selector center
			Local dx:Int = SEL_RAD -x
			Local dy:Int = SEL_RAD -y
			Local d:Int = Sqr(dx*dx+dy*dy)
			' set color according to distance (default: transparent)
			Local argb:Int = 0
			' outer border
			If d = SEL_RAD Then
				argb = SEL_OUTER
			' inner border
			ElseIf d = SEL_RAD-1 Then
				argb = SEL_INNER
			EndIf
			WritePixel(pm, x,y, argb)
		Next
	Next
	UnlockImage(imgSel)
End Function

' update color and all indicating gadgets
Function UpdateCol()
	' restrict selection to 0 ... FIELD_MAX
	selx = Max(0, Min(FIELD_MAX, selx))
	sely = Max(0, Min(FIELD_MAX, sely))
	' interpolate color for selection
	selrgb = InterpolCol(selx, sely)
	' get color components
	Local cs:Int[3]
	For Local i:Int = 0 Until 3
		Local c:Int = (selrgb Shr (16-i*8)) & $ff
		' update gadgettexts
		SetGadgetText(gdCol[i], c)
		cs[i] = c
	Next
	' update hex indicator
	Local txt:String = Right(Hex(selrgb), 6)
	SetGadgetText(gdCol[3], txt)
	' update color indicator
	SetGadgetColor(gdColInd, cs[RED], cs[GRN], cs[BLU])
End Function

' interpolate between the four base colors for x / y
Function InterpolCol:Int(pX:Int, pY:Int)
	Local col:Int[3]
	' relative position
	Local fy:Float = pY /Float(FIELD_MAX)
	Local gy:Float = 1 -fy
	Local fx:Float = pX /Float(FIELD_MAX)
	Local gx:Float = 1 -fx
	' interpolate color
	For Local ic:Int = 0 Until 3
		col[ic] = bcol[0][ic]*gx*gy +bcol[1][ic]*fx*gy +bcol[2][ic]*gx*fy +bcol[3][ic]*fx*fy
	Next
	' build color
	Return col[RED] Shl 16 | col[GRN] Shl 8 | col[BLU]
End Function